; Simple PAL TV frame generator PIC 16f628 code.. V 1.03
; Displays 6-bit colour bars using simple resistor DACs - see schematic
; By Phil Ruston '05
; Use 20MHz crystal 0.2 uS per instruction
;-----------------------------------------------------------------------------

	list	p=16f628           	; list directive to define processor
	list	r=decimal

	#include <p16f628.inc>        	; processor specific variable definitions

	#define bank0             bcf STATUS,RP0               ;macros
	#define bank1             bsf STATUS,RP0
	#define skipifzero        btfss STATUS,Z        
	#define skipifnotzero     btfsc STATUS,Z
	#define skipifcarry       btfss STATUS,C        
	#define skipifnotcarry    btfsc STATUS,C
	
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_ON & _HS_OSC & _MCLRE_OFF & _LVP_OFF

;----- PROJECT EQUATES -----------------------------------------------------------------------

; PORT A

tv_sync			 	equ 0			; output - sync 

; PORT B

; 6 bit colour data on bits 0:5.. 0:1=Red/2:3=Green/4:5=Blue

;---- Constants ------

top_border 			equ 28			; thickness of top border in lines
y_window			equ 266			; size of ydisplay in lines (must be even)
bottom_border 		equ 10			; thickness of bottom border in lines  
;                 					  (must add up to 304 lines)

;----- VARIABLES------------------------------------------------------------------------------

loops               equ 0x20     	; first address of user reg space (bank 0)
wait_time           equ 0x21
frame_count         equ 0x22
temp				equ 0x23
interlace			equ 0x24		; bit 0 toggles interlace on/off
output_loop			equ 0x25

;*********************************************************************************************

			ORG 0x000     			; processor reset vector
	
			goto main  		  	  	; go to beginning of program

			ORG 0x004 				; interrupt vector location

			retfie            	  	; return from interrupt

;---------------------------------------------------------------------------------------------
							
main		movlw b'00000111'		; INITIALIZE CHIP	
			movwf CMCON				; use digital mode for PORTB (disable comparitors)
			clrf PORTA
			clrf PORTB
			banksel TRISA
			movlw b'00000000'		; set data direction for port A
			movwf TRISA
			movlw b'00000000'		; set data direction for port B
			movwf TRISB
			banksel PORTA

;--------------------------------------------------------------------------------------------

my_init		clrf PORTB
			clrf frame_count
			clrf interlace

;--------------------------------------------------------------------------------------------

display		movlw top_border-1  	;1					-
			movwf loops         	;1					!
borderloop1	call blank_line      	;2 + 313 = 315		!
			nop						;1					!
			nop						;1					!  TOP BORDER
			decfsz loops,f			;1/2				!
			goto borderloop1		;2					!
			nop						;1					!
			call blank_line			;2 + 313 = 315		!
			nop						;1					!
			nop						;1					!
			nop						;1					-		

			movlw (y_window/2)-1	;1					-
			movwf loops				;1					!	
ywinaloop	call data_line			;2 + 313 = 315 		!	
			nop						;1					!	
			nop						;1					!	
			decfsz loops,f			;1/2				!
			goto ywinaloop			;2/0				!
			nop						;1					!
			call data_line			;2					!					
			movlw (y_window/2)-1	;1					!					
			btfsc interlace,0		;2/1				!		
			addlw 1					;0/1				!		
			movwf loops				;1					!	Y DISPLAY WINDOW		
			nop 					;1					!				
ywinbloop	call data_line			;2 + 313 = 315 		!					
			nop						;1					!			
			nop						;1					!						
			decfsz loops,f			;1/2				!				
			goto ywinbloop			;2/0				!				
			nop						;1					!		
			call data_line			;2 + 313 = 315		!			
			nop						;1					!				
			nop						;1					!			
			nop						;1					-			

			movlw bottom_border-1 	;1					-
			movwf loops         	;1					!
borderloop2	call blank_line      	;2 + 313 = 315		!
			nop						;1					!	
			nop						;1					!
			decfsz loops,f			;1/2				! BOTTOM BORDER
			goto borderloop2		;2					!
			nop						;1					!
			call blank_line			;2 + 313 = 315		!	
			nop						;1					!
			nop						;1					-

			btfsc frame_count,0		;1/2				-
			goto field2vsync		;2/0				!
			nop						;1					!		
field1vsync	call short_sync			;2+158				!
			call short_sync			;""					!
			call short_sync			;""					!
			call short_sync			;""					!						
			call short_sync			;""					!	
			call short_sync			;""					!	
			call long_sync			;""					!									
			call long_sync			;""					!		
			call long_sync			;""					!			
			call long_sync			;""					!										
			call long_sync			;""					!				
			call short_sync			;""					!				
			call short_sync			;""					!		
			call short_sync			;""					!			
			call short_sync			;""					!			
			call sh_sync_b			;2+152				!			
			btfsc interlace,0		;1					!			
			incf frame_count,f		;1					!  VERTICAL SYNC			
			goto display			;2					!			
field2vsync	call short_sync			;2+158				!				
			call short_sync			;""					!			
			call short_sync			;""					!			
			call short_sync			;""					!			
			call short_sync			;""					!						
			call long_sync			;""					!						
			call long_sync			;""					!															
			call long_sync			;""					!						
			call long_sync			;""					!							
			call long_sync			;""					!															
			call short_sync			;""					!							
			call short_sync			;""					!																	
			call short_sync			;""					!							
			call sh_sync_b			;2+152				!								
			btfsc interlace,0		;1					!						
			incf frame_count,f		;1					!						
			goto display			;2					-				

;========== Called routines =======================================================================

blank_line	bcf PORTA,tv_sync		;1
			movlw 3					;
			call wait_n_usec		;15
			nop						;1
			nop						;1
			nop						;1
			nop						;1
			bsf PORTA,tv_sync		;1
			movlw 8					;
			call wait_n_usec		;40
			movlw 50				;
			call wait_n_usec		;250
			return					;2	(313 total)	

;----------------------------------------------------------------------------------------------------

data_line	bcf PORTA,tv_sync		;1
			movlw 3					;
			call wait_n_usec		;15
			nop						;1
			nop						;1
			nop						;1
			nop						;1
			bsf PORTA,tv_sync		;1
			movlw 8					;
			call wait_n_usec		;40 (61 total up to here) 
			
			movlw 62				;1
			movwf output_loop		;1
do_colours	incf PORTB,f			;1
			decfsz output_loop,f	;1/2
			goto do_colours			;2  (loop = 247)
			clrf PORTB				;1  
			return					;2	(313 total)	

;----------------------------------------------------------------------------------------------------

short_sync	bcf PORTA,tv_sync		;1
			nop						;1
			nop						;1
			nop						;1
			nop						;1
			nop						;1
			nop						;1
			nop						;1
			nop						;1
			nop						;1
			bsf PORTA,tv_sync		;1
			movlw 29				;
			call wait_n_usec		;145
			return					;2	(158 total)


sh_sync_b	bcf PORTA,tv_sync		;1
			nop						;1
			nop						;1
			nop						;1
			nop						;1
			nop						;1
			nop						;1
			nop						;1
			nop						;1
			nop						;1
			bsf PORTA,tv_sync		;1
			movlw 27				;
			call wait_n_usec		;135
			nop						;1
			nop						;1
			nop						;1
			nop						;1
			return					;2	(152 total)

;----------------------------------------------------------------------------------------------------

long_sync	bcf PORTA,tv_sync		;1
			movlw 29				;
			call wait_n_usec		;145
			nop						;1
			nop						;1
			nop						;1
			nop						;1
			bsf PORTA,tv_sync		;1
			nop						;1
			nop						;1
			nop						;1
			nop						;1
			nop						;1
			return					;2 (158 total)

;--------------------------------------------------------------------------------

wait_n_usec

			addlw 254				;1  (minimum wait = 3 usec!)
			movwf temp				;1
waitloop	nop						;1
			nop						;1
			decfsz temp,f			;1/2
			goto waitloop			;2/0
			nop						;1
			nop						;1
			nop						;1
			nop						;1
			return					;2

;********************************************************************************

			END                       ; directive 'end of program'
